/*
 * File         : BV_Session_Control.h
 * Author       : Tomasz Stachowiak
 * Created      : Thu May 21 20:01:17 1998
 * Revision     : $Revision$
 * Updated      : $Date$
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1998
 */

#ifndef __BV_SESSION_CONTROL_H
#define __BV_SESSION_CONTROL_H

#include <map.h>
#include <BVN/Reactor.h>
#include "BV_Member.h"
#include "BV.h"

//---GUI callbacks
#include "MainForm.h"
class MainWindowMainWindow;
//---GUI callbacks END

#define ID_NULL 255

  typedef enum
    {
      _init,
      _invited,
      _accepting,
      _session
    } BV_Status;

class BV_Session_Control
{

 public:

  static BV_Session_Control *instance();
  static void destroy();
  static int reference_no_;
  static BV_Session_Control *session_control_;

  BV_Session_Control();
  ~BV_Session_Control();
  void set_reactor(Reactor *reactor);
  // network callbacks
  int invited(BV_Member *member);
  int accepted(char *name, int host);
  int rejected(char *name, int host);
  int confirmed(char *msg);
  int connect(int id, SOCK_Stream *stream);
  int new_member(BV_Member *member);
  int delete_member(int id);
  int connection_closed(SOCK_Stream *stream);
  void app_message(int user_id, char *msg, int len);

  // user requests
  int invite(char *name, char *host);
  int accept_invitation();
  int reject_invitation();
  int quit();
  int reset();
  int send_to_app(int no, int tag, char *msg, int len);
  
  Options *get_options();
  void update_options();
  int get_local_id();
  map <int, BV_Member> *get_member_list();  
  BV_Member *get_member(int id);
  
  //---GUI callbacks
  void setGUI(MainWindowMainWindow *wnd, MainForm *form);
  MainForm *getMainForm();
  //---GUI callbacks END

 private:

  void send_to_all(BV_Message *msg);
  void fill_participants(BV_Message *msg);
  int restore_options();
  int save_options();
  
  map <int, BV_Member> members_;
  Status status_;
  BV_Member *inviting_member_;
  BV_Member *new_member_;
  unsigned char local_id_;
  int last_id_;
  int internal_id_;
  Reactor *reactor_;

  Options options_;

  //---GUI callbacks
  MainForm *form_;
  MainWindowMainWindow *main_wnd_;
  //---GUI callbacks END

};

#endif

